/**
 * (c) Copyright 2000-2012 spring saarlouis
 * All Rights Reserved.
 * 
 * contact info@spring.de for more information
 */
package de.spring.mobile;

import android.app.Activity;
import android.content.Context;
import android.os.Bundle;
import android.util.Log;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Map;

/**
 * This activity can be used to measure streaming contents within it.
 * by the {@link SpringStreams} sensor.
 * 
 * @see SpringStreams
 * @see StreamAdapter
 * @see Stream
 * 
 * @author  <a href="mailto:support@spring.de">spring GmbH &amp; Co.KG</a>
 */
public abstract class SpringStreamsActivity extends Activity {

	private static SpringStreams sensor;
	private static int instances = 0;
	private static String appName = null;
	private static String site = null;
	private static String market = null;

	private Stream stream;

	/**
	 * Instantiate the class with the given <code>site</code> and <code>appName</code>.
	 * 
	 * @param site The site name. It will be delivered by the measurement provider.
	 * @param appName The application name. It will be delivered by the measurement provider.
	 * 
	 * @see #SpringStreamsActivity(String, String)
	 * 
	 * @throws NullPointerException If a parameter is <code>null</code>
	 * @throws IllegalArgumentException If the timeout is less than 5 seconds.
	 */
	public SpringStreamsActivity(String site, String appName) {
		if(site == null) throw new NullPointerException("parameter site may not be null");
		if(appName == null) throw new NullPointerException("parameter appName may not be null");

		if(SpringStreamsActivity.site == null) SpringStreamsActivity.site = site;
		if(SpringStreamsActivity.appName == null) SpringStreamsActivity.appName = appName;
	}

	/**
	 * Instantiate the class with the given <code>site</code> and <code>appName</code>.
	 *
	 * @param site The site name. It will be delivered by the measurement provider.
	 * @param appName The application name. It will be delivered by the measurement provider.
	 * @param market The market name. It will be delivered by the measurement provider.
	 *
	 * @see #SpringStreamsActivity(String, String, String)
	 *
	 * @throws NullPointerException If a parameter is <code>null</code>
	 * @throws IllegalArgumentException If the timeout is less than 5 seconds.
	 */
	public SpringStreamsActivity(String site, String appName, String market) {
		if(site == null) throw new NullPointerException("parameter site may not be null");
		if(appName == null) throw new NullPointerException("parameter appName may not be null");
		if(market == null) throw new NullPointerException("parameter market may not be null");

		if(SpringStreamsActivity.site == null) SpringStreamsActivity.site = site;
		if(SpringStreamsActivity.appName == null) SpringStreamsActivity.appName = appName;
		if(SpringStreamsActivity.market == null) SpringStreamsActivity.market = market;
	}

	/**
	 *
	 * @throws InvocationTargetException
	 * @throws IllegalAccessException
	 */
	private void init() throws InvocationTargetException, IllegalAccessException {
		if(sensor == null) {
			Method m1 = null, m2 = null;
			try {
				m1 = SpringStreams.class.getDeclaredMethod("getInstance", String.class, String.class, Context.class);
			} catch (Exception e) {}
			try {
				m2 = SpringStreams.class.getDeclaredMethod("getInstance", String.class, String.class, Context.class, String.class);
			} catch (Exception e) {}
			Context context = getApplicationContext();
			if(context != null) {
				if(m1 != null){
					sensor = (SpringStreams) m1.invoke(null, site, appName, getApplicationContext());
				} else if(m2 != null){
					sensor = (SpringStreams) m2.invoke(null, site, appName, getApplicationContext(), market);
				}
			} else Log.e("SpringStreamsActivity", "getApplicationContext() returns NULL");
		}
	}
	
	/**
	 * Returns the underlying {@link Stream}-Object which handles the measured stream.
	 * 
	 * @return The {@link Stream}
	 */
	public Stream getStream() {
		return stream;
	}

	/**
	 * Set the timeout on the sensor. 
	 * 
	 * @throws IllegalArgumentException If the timeout is less than 5 seconds
	 * 
	 * @param timeout The timeout in seconds.
	 */
	public void setTimeout(int timeout) {
		if(timeout < 5) throw new IllegalArgumentException("timeout less than 5 seconds.");
		sensor.setTimeout(timeout);
	}
	
	/**
	 * Return the timeout.
	 * 
	 * @return The timeout
	 */
	public int getTimeout() {
		return sensor.getTimeout();
	}
	
	/**
	 * Set the debug property on the sensor.
	 * 
	 * @see SpringStreams#setDebug(boolean)
	 * 
	 * @param debug Enables or disables sensor debugging.
	 */
	public void setSensorDebug(boolean debug) {
		sensor.setDebug(debug);
	}
	
	/**
	 * Set the offline mode on or off.
	 * 
	 * @param offlineMode Enables or disables the offline mode.
	 */
	public void setOfflineMode(boolean offlineMode) {
		sensor.setOfflineMode(offlineMode);
	}
	
	/**
	 * Returns <code>true</code> if offline mode is on otherwise <code>false</code>.
	 * 
	 * @return Returns <code>true</code> if offline mode is on otherwise <code>false</code>.
	 */
	public boolean isOfflineMode() {
		return sensor.isOfflineMode();
	}
	
	/**
	 * Returns the debug property on the sensor.
	 * 
	 * @see SpringStreams#isDebug()
	 * 
	 * @return Returns the debug property on the sensor.
	 */
	public boolean isSensorDebug() {
		return sensor.isDebug();
	}
	
	/**
	 * Set the tracking property on the sensor.
	 * 
	 * @see SpringStreams#setTracking(boolean)
	 * 
	 * @param tracking Enables or disables the tracking feature.
	 */
	public void setTracking(boolean tracking) {
		sensor.setTracking(tracking);
	}
	
	/**
	 * Returns the tracking property on the sensor.
	 * 
	 * @see SpringStreams#isTracking()
	 * 
	 * @return Returns the tracking property on the sensor.
	 */
	public boolean isTracking() {
		return sensor.isTracking();
	}	

	/**
	 * Calls internally the {@link SpringStreams#track(StreamAdapter, Map)} method.
	 * 
	 * @see SpringStreams#track(StreamAdapter, Map)
	 * 
	 * @param adapter The adpater
	 * @param atts The attribute map
	 */
	public void track(StreamAdapter adapter, Map<String, Object> atts) {
		if(this.stream != null) {
			stream.stop();
			stream = null;
		}
		stream = sensor.track(adapter, atts);
	}

	/**
	 * Counts internally all created {@link SpringStreamsActivity} instances.
	 * 
	 * @see android.app.Activity#onCreate(android.os.Bundle)
	 */
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		try {
			init();
		} catch (InvocationTargetException | IllegalAccessException e) {
			e.printStackTrace();
		}
		instances++;
		super.onCreate(savedInstanceState);
	}

	/**
	 * Stops internally the measurement by calling the method
	 * {@link Stream#stop()}.
	 * 
	 * @see Stream#stop()
	 * @see android.app.Activity#onStop()
	 */
	@Override
	protected void onStop() {
		if(stream != null) {
			stream.stop();
			stream = null;
		}
		super.onStop();
	}
	
	/**
	 * If the last instance of {@link SpringStreamsActivity} is destroying
	 * the method {@link SpringStreams#unload()} will be called.
	 * 
	 * @see SpringStreams#unload()
	 * @see android.app.Activity#onDestroy()
	 */
	@Override
	protected void onDestroy() {
		instances--;
		if(instances <= 0) {
			sensor.unload();
			sensor = null;
			stream = null;
		}
		super.onDestroy();
	}

}
